/* Beispiele aus dem Buch "LDAP fuer Java-Entwickler", 
 * Stefan Zoerner, 4. aktualisierte Auflage 2013
 *  
 * Webseite zum Buch: http://www.ldapjava.de/ 
 * Software & Support Media GmbH / entwickler.press, http://www.entwickler-press.de/
 */
package ldapbuch.kapitel5.ad;

import javax.naming.Context;
import javax.naming.NamingEnumeration;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.naming.directory.ModificationItem;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;

/**
 * Kennwort eines Benutzers &auml;ndern -- (Listing 5_5).
 * 
 * Verbindungsdaten werden teilweise von aussen erwartet (jndi.properties),
 * teilweise als Kommandozeilenparameter. Das Beispiel setzt
 * SSL-Verschl&uuml;sselung und die Verbindung eines Nutzers mit ausreichenden
 * Rechten zum &Auml;ndern des Kennwortes voraus.
 * 
 * @author Stefan Z&ouml;rner
 */
public class KennwortImAdAendern {

  static String FILTER_USERS = "(&(objectclass=user)(sAMAccountName={0}))";

  /**
   * Einstiegspunkt. Als Kommandozeilenparameter werden als erste beiden Werte
   * die Credentials zur Verbindungsaufnahme erwartet, gefolgt von der Kennung
   * (sAMAccountName) des betroffenen Benutzers, und altem und neuem Kennwort.
   * 
   * @param args
   *          Kommandozeilenparameter
   */
  public static void main(String[] args) throws Exception {

    // Authentifizierung
    InitialDirContext ctx = new InitialDirContext();
    ctx.addToEnvironment(Context.SECURITY_AUTHENTICATION,
        "simple");
    ctx.addToEnvironment(Context.SECURITY_PRINCIPAL, args[0]);
    ctx.addToEnvironment(Context.SECURITY_CREDENTIALS, args[1]);

    // Eintrag des Benutzers suchen
    String userId = args[2];
    SearchControls ctls = new SearchControls();
    ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
    ctls.setReturningObjFlag(true);

    NamingEnumeration<SearchResult> enm = ctx.search("",
        FILTER_USERS, new String[] { userId }, ctls);

    if (!enm.hasMore()) {
      System.err.println("User nicht im Verzeichnis gefunden.");
    } else {
      SearchResult result = enm.next();
      DirContext userCtx = (DirContext) result.getObject();

      // Werte fuer Kennwoerter im AD erzeugen
      String kennwortAlt = args[3];
      String kennwortAltAD = "\"" + kennwortAlt + "\"";
      byte bytesAlt[] = kennwortAltAD.getBytes("UTF-16LE");
      String kennwortNeu = args[4];
      String kennwortNeuAD = "\"" + kennwortNeu + "\"";
      byte bytesNeu[] = kennwortNeuAD.getBytes("UTF-16LE");

      // Aenderung durchfuehren
      ModificationItem mods[] = new ModificationItem[2];
      mods[0] = new ModificationItem(DirContext.REMOVE_ATTRIBUTE,
          new BasicAttribute("unicodePwd", bytesAlt));
      mods[1] = new ModificationItem(DirContext.ADD_ATTRIBUTE,
          new BasicAttribute("unicodePwd", bytesNeu));
      userCtx.modifyAttributes("", mods);
    }

    ctx.close();
  }
}