/* Beispiele aus dem Buch "LDAP fuer Java-Entwickler", 
 * Stefan Zoerner, 4. aktualisierte Auflage 2013
 *  
 * Webseite zum Buch: http://www.ldapjava.de/ 
 * Software & Support Media GmbH / entwickler.press, http://www.entwickler-press.de/
 */
package ldapbuch.kapitel5.ad;

import java.io.UnsupportedEncodingException;

import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.BasicAttributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;

/**
 * Einen Dom&auml;nenbenutzer im AD anlegen -- (Listing 5_6).
 * 
 * Verbindungsdaten (Host, Base DN) werden von aussen erwartet
 * (jndi.properties). Das Beispiel setzt SSL-Verschl&uuml;sselung und die
 * Verbindung eines Nutzers mit ausreichenden Rechten zum Anlegen von Benutzern
 * voraus.
 * 
 * @author Stefan Z&ouml;rner
 */
public class NeuenBenutzerAnlegen {

  /**
   * Einstiegspunkt. Als Kommandozeilenparameter werden die Credentials zur
   * Verbindungsaufnahme erwartet (Benutzer, Kennwort). Die Daten des
   * anzulegenden Benutzers sind hart verdrahtet.
   * 
   * @param args
   *          Kommandozeilenparameter
   */
  public static void main(String[] args) throws NamingException,
      UnsupportedEncodingException {

    // Verbindung aufbauen
    DirContext ctx = new InitialDirContext();
    ctx.addToEnvironment(Context.SECURITY_AUTHENTICATION, "simple");
    ctx.addToEnvironment(Context.SECURITY_PRINCIPAL, args[0]);
    ctx.addToEnvironment(Context.SECURITY_CREDENTIALS, args[1]);

    // Eintrag erstellen
    //
    BasicAttributes as = new BasicAttributes();
    BasicAttribute ocs = new BasicAttribute("objectclass");
    ocs.add("top");
    ocs.add("person");
    ocs.add("organizationalPerson");
    ocs.add("user");
    as.put(ocs);
    as.put(new BasicAttribute("givenName", "Paulchen"));
    as.put(new BasicAttribute("sn", "Panther"));
    as.put(new BasicAttribute("cn", "Paulchen Panther"));
    as.put(new BasicAttribute("displayName", "Paulchen Panther"));
    as.put(new BasicAttribute("samaccountname", "ppanther"));
    as.put(new BasicAttribute("userPrincipalName", "ppanther@bundestag.de"));
    as.put(new BasicAttribute("description", "Der rosarote Panther"));
    as.put(new BasicAttribute("mail", "ppanther@bundestag.de"));
    as.put(new BasicAttribute("wWWHomePage", "http://de.wikipedia.org/wiki/Der_rosarote_Panther"));
    as.put(new BasicAttribute("userAccountControl", ""
        + (AdsUserFlags.ADS_UF_NORMAL_ACCOUNT)));

    // Attribute fuer das Kennwort
    String kennwort = "Geheim1!";
    String kennwortAD = "\"" + kennwort + "\"";
    byte bytes[] = kennwortAD.getBytes("UTF-16LE");
    as.put(new BasicAttribute("unicodePwd", bytes));
    as.put(new BasicAttribute("pwdLastSet", "-1"));

    // Eintrag anlegen
    ctx.createSubcontext("cn=Paulchen Panther,OU=personen", as);
  }
}